/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2016.01.03.
*   Chip         -  Atmel AMega8
*   Compiler     -  avr-gcc ( WinAVR)
*
*   LED VU mero
*
********************************************************************************
*   PonyProg Configuration and Security Bits (bepipalva):
*
*   CKSEL3, CKSEL2, CKSEL1, CKSEL0
*   P       P       -       -       0011  Calibrated Internal RC Oscillator 4MHz
*
*   Calibrated Internal RC Oscillator
*   SUT1  SUT0
*   -     P         Slowly rising power
*   
*******************************************************************************/
#define F_CPU                _4MHZ     // orajel 4MHz


#include "tkiraaly_atmega8.h"
#include <util/delay.h>


#define ROW_LEFT_LOW_ON      PB7_1
#define ROW_RIGHT_LOW_ON     PB6_1
#define ROW_RIGHT_HIGH_ON    PC5_1
#define ROW_LEFT_HIGH_ON     PC4_1

#define LED0_ON              PC3_1
#define LED1_ON              PC2_1
#define LED2_ON              PC1_1
#define LED3_ON              PC0_1
#define LED4_ON              PB2_1
#define LED5_ON              PB1_1
#define LED6_ON              PB0_1
#define LED7_ON              PD7_1
#define LED8_ON              PD6_1
#define LED9_ON              PD5_1
#define LEDA_ON              PD4_1
#define LEDB_ON              PD3_1
#define LEDC_ON              PD2_1
#define LEDD_ON              PD1_1
#define LEDE_ON              PD0_1

#define LEFT_IN              ADC_IN7
#define RIGHT_IN             ADC_IN6

#define MEASURE_PERIOD       25
#define PEAK_PERIOD          25



void led_bar_low( U8);                 // LED oszlop kijelzes also reszen
void led_bar_high( U8);                // LED oszlop kijelzes felso reszen
void led_point_low( U8);               // LED pont kijelzes also reszen
void led_point_high( U8);              // LED pont kijelzes felso reszen




int main( void)
{
   U8 row_count= 3;                    // kijelzo sor valtas
   U8 measure_count= 0;                // meres szamlalo
   U8 left_nright= 0;                  // bal / nem jobb - oldal valtas
   U8 left_u= 0;                       // bal feszultseg
   U8 left_u_peak= 0;                  // bal csucs feszultseg
   U8 left_count= 0;                   // bal szamlalo
   U8 right_u= 0;                      // jobb feszultseg
   U8 right_u_peak= 0;                 // jobb csucs feszultseg
   U8 right_count= 0;                  // jobb szamlalo

   ADC_8BIT;                           // 8 bit felbonas
   ADC_VREF_INT;                       // belso 2V56 ref.fesz, nincs szuro kondi
   ADC_IN5;                            // 5. bemenet
   ADC_FREE_RUN;                       // folyamatos meres

   PB7_OUT;                            // ROWs 0..3
   PB6_OUT;
   PC5_OUT;
   PC4_OUT;

   PD0_OUT;                            // LEDs 0..14
   PD1_OUT;
   PD2_OUT;
   PD3_OUT;
   PD4_OUT;
   PD5_OUT;
   PD6_OUT;
   PD7_OUT;

   PC0_OUT;
   PC1_OUT;
   PC2_OUT;
   PC3_OUT;

   PB0_OUT;
   PB1_OUT;
   PB2_OUT;

   for(;;)
   {
      PORTB= 0;                             // torles
      PORTC= 0;
      PORTD= 0;
      if( row_count) row_count--;
      else row_count= 3;
      switch( row_count)                    // kijelzes
      {
         case 0:
                 led_bar_high( left_u);
                 led_point_high( left_u_peak);
                 ROW_LEFT_HIGH_ON;
                 break;
         case 1:
                 led_bar_low( left_u);
                 led_point_low( left_u_peak);
                 ROW_LEFT_LOW_ON;
                 break;
         case 2:
                 led_bar_high( right_u);
                 led_point_high( right_u_peak);
                 ROW_RIGHT_HIGH_ON;
                 break;
         case 3:
                 led_bar_low( right_u);
                 led_point_low( right_u_peak);
                 ROW_RIGHT_LOW_ON;
                 break;
      }

/* ez egy kijelzo teszt, vegiglepteti a LED-eket
   hasznalatakor a measure modult ki kell REM-elni

   if ( measure_count) measure_count--;
   else
   {
      measure_count= 255;      
      if( left_u_peak++ > 30) left_u_peak= 0; 
//      if( left_u++ > 30) left_u= 0; 
//      if( right_u_peak++ > 30) right_u_peak= 0; 
//      if( right_u++ > 30) right_u= 0; 
   }
*/

      if ( measure_count) measure_count--;  // meres
      else
      {
         measure_count= MEASURE_PERIOD;      
         if( left_count) left_count--;      // csucs eretekek idozitese
         else left_u_peak= 0;
         if( right_count) right_count--;
         else right_u_peak= 0;
         if ( left_nright)
         {                                  // bal meres
            left_u= ( ADCH >> 2) + 1;
            if ( left_u> 30) left_u= 30;
            if ( left_u >= left_u_peak)
            {
               left_u_peak= left_u;
               left_count= PEAK_PERIOD;
            }                 
            left_nright= 0;
            RIGHT_IN;
         }
         else
         {                                  // jobb meres
            right_u= ( ADCH >> 2) + 1;
            if ( right_u> 30) right_u= 30;
            if ( right_u >= right_u_peak)
            {
               right_u_peak= right_u;
               right_count= PEAK_PERIOD;
            }
            left_nright= 1;
            LEFT_IN;
         }
      }
      _delay_ms( 1);
   }
}




void led_bar_low( U8 n)                // LED oszlop also reszen 
{
   if( n) {LED0_ON; n--;}   
   if( n) {LED1_ON; n--;}   
   if( n) {LED2_ON; n--;}   
   if( n) {LED3_ON; n--;}   
   if( n) {LED4_ON; n--;}   
   if( n) {LED5_ON; n--;}   
   if( n) {LED6_ON; n--;}   
   if( n) {LED7_ON; n--;}   
   if( n) {LED8_ON; n--;}   
   if( n) {LED9_ON; n--;}   
   if( n) {LEDA_ON; n--;}   
   if( n) {LEDB_ON; n--;}   
   if( n) {LEDC_ON; n--;}   
   if( n) {LEDD_ON; n--;}   
   if( n) {LEDE_ON; n--;}   
}




void led_bar_high( U8 n)               // LED oszlop felso reszen
{
   if( n > 15) 
   {
      n-= 15;
      if( n) {LEDE_ON; n--;}   
      if( n) {LEDD_ON; n--;}   
      if( n) {LEDC_ON; n--;}   
      if( n) {LEDB_ON; n--;}   
      if( n) {LEDA_ON; n--;}   
      if( n) {LED9_ON; n--;}   
      if( n) {LED8_ON; n--;}   
      if( n) {LED7_ON; n--;}   
      if( n) {LED6_ON; n--;}   
      if( n) {LED5_ON; n--;}   
      if( n) {LED4_ON; n--;}   
      if( n) {LED3_ON; n--;}   
      if( n) {LED2_ON; n--;}   
      if( n) {LED1_ON; n--;}   
      if( n) {LED0_ON; n--;}   
   }
}




void led_point_low( U8 n)              // LED pont also reszen
{
   switch( n)
   {
      case  1: LED0_ON; break;
      case  2: LED1_ON; break;
      case  3: LED2_ON; break;
      case  4: LED3_ON; break;
      case  5: LED4_ON; break;
      case  6: LED5_ON; break;
      case  7: LED6_ON; break;
      case  8: LED7_ON; break;
      case  9: LED8_ON; break;
      case 10: LED9_ON; break;
      case 11: LEDA_ON; break;
      case 12: LEDB_ON; break;
      case 13: LEDC_ON; break;
      case 14: LEDD_ON; break;
      case 15: LEDE_ON;
      default: break;
   }
}




void led_point_high( U8 n)             // LED pont felso reszen
{
   switch( n- 15)
   {
      case  1: LEDE_ON; break;
      case  2: LEDD_ON; break;
      case  3: LEDC_ON; break;
      case  4: LEDB_ON; break;
      case  5: LEDA_ON; break;
      case  6: LED9_ON; break;
      case  7: LED8_ON; break;
      case  8: LED7_ON; break;
      case  9: LED6_ON; break;
      case 10: LED5_ON; break;
      case 11: LED4_ON; break;
      case 12: LED3_ON; break;
      case 13: LED2_ON; break;
      case 14: LED1_ON; break;
      case 15: LED0_ON;
      default: break;
   }
}
